#!/usr/bin/perl
#
################################################################################
#
# IPCop CopTime (F)Cron Job
#
# Copyright (C) 2007-2008 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA 
#
#
# Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der GNU General Public License, wie von der Free Software Foundation verffentlicht, weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder (nach Ihrer Option) jeder spteren Version. 
#
# Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
#
# Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: coptime.pl 310 2008-05-26 18:43:38Z weizen_42 $
#
# Revision history:
#
# 2007-02-xx created by weizen_42 at ipcop-forum dot de
# 2007-02-21 v0.1.0 field for image
# 2007-02-25 v0.2.0 send password as md5hash, add field for galerie (http://www.ipcop-forum.de/galerie/index.php)
# 2007-03-12 v0.3.0 change timing
#

use strict;

use IO::Socket;
use LWP::UserAgent;
use POSIX qw(strftime);
use Digest::MD5  qw(md5_hex);

require '/var/ipcop/general-functions.pl';

my $debug = 0;
my $fcron = 0;
my $version = 'v0.4.2';
my $server_url = 'http://www.ipcop-forum.de/cgi-bin/coptime.cgi';
my %data = ();
my $ua = LWP::UserAgent->new;
my $content;


while ( @ARGV )
{
  my $arg = shift @ARGV;

  $debug = 1 if ( $arg eq 'debug' );
  $fcron = 1 if ( $arg eq 'fcron' );
}


fetch_settings();
exit 0 unless ( $data{'ENABLED'} eq 'on' );

# no point if disconnected
unless ( -e "${General::swroot}/red/active" )
{
  open(FILE, ">/var/log/coptime/status");
  print FILE "STATUS=Error, not connected\n";
  print FILE "CONTACT=". strftime("%F %H:%M", localtime) ."\n";
  close(FILE);
  exit(0);
}

prepare_ua();

if ( $fcron > 0 )
{
  fetch_data();

  for( my $i = 0; $i < 5; $i++ )
  {
    $data{'UPTIME_ACTUAL'} = `/bin/cat /proc/uptime | cut -f 1 -d ' '`;
    $content = $ua->post($server_url, 
                            [ uptime_id       => $data{'UPTIME_ID'},
                              password        => md5_hex($data{'PASSWORD'}),
                              image           => $data{'IMAGE'},
                              galerie         => $data{'GALERIE'},
                              uptime_actual   => $data{'UPTIME_ACTUAL'},
                              ipcop_version   => $data{'IPCOP_VERSION'},
                              os_name         => $data{'OS_NAME'},
                              os_version      => $data{'OS_VERSION'},
                              machine         => $data{'MACHINE'},
                            ]);

    if ( $content->is_success )
    {
      print $content->content if ( $debug > 0 );

      # store actual info
      open(FILE, ">/var/log/coptime/status");
      
      print FILE "STATUS=$1\n"  if ( $content->content =~ /.*status : (.*)/ );
      print FILE "CONTACT=". strftime("%F %H:%M:%S UTC%z", localtime) ."\n";
      print FILE "UPTIME=$1\n"  if ( $content->content =~ /.*uptime_actual : (.*)/ );
      print FILE "RECORD=$1\n"  if ( $content->content =~ /.*uptime_record : (.*)/ );
      print FILE "AVERAGE=$1\n" if ( $content->content =~ /.*uptime_average : (.*)/ );
      print FILE "RANK=$1\n"    if ( $content->content =~ /.*rank : (.*)/ );
        
      close(FILE);

      exit 0;
    }
    else
    {
      print $content->content if ( $debug > 0 );
      sleep(15);
    }
  }

  print "No connection, giving up\n" if ( $debug > 0 );

  open(FILE, ">/var/log/coptime/status");
  print FILE "STATUS=Error, no connection to server\n";
  print FILE "CONTACT=". strftime("%F %H:%M", localtime) ."\n";
  close(FILE);
  exit 1;
}

exit 0;


sub prepare_ua()
{
  $ua->timeout(60);
  $ua->agent("Mozilla/4.0 (compatible; IPCop $General::version; $version)");

  # upstream proxy
  my %proxysettings=();
  &General::readhash("${General::swroot}/proxy/settings", \%proxysettings);

  if ( $_ = $proxysettings{'UPSTREAM_PROXY'} )
  {
    my ($peer, $peerport) = (/^(?:[a-zA-Z ]+\:\/\/)?(?:[A-Za-z0-9\_\.\-]*?(?:\:[A-Za-z0-9\_\.\-]*?)?\@)?([a-zA-Z0-9\.\_\-]*?)(?:\:([0-9]{1,5}))?(?:\/.*?)?$/);
    if ( $proxysettings{'UPSTREAM_USER'} )
    {
      $ua->proxy("http","http://$proxysettings{'UPSTREAM_USER'}:$proxysettings{'UPSTREAM_PASSWORD'}@"."$peer:$peerport/");
    } 
    else 
    {
      $ua->proxy("http","http://$peer:$peerport/");
    }
  }
}


sub fetch_settings()
{
  # fetch uptime_id, password, etc.
  $data{'ENABLED'} = '';
  $data{'UPTIME_ID'} = 0;
  $data{'PASSWORD'} = '';
  $data{'IMAGE'} = '';
  $data{'GALERIE'} = '';
  &General::readhash("${General::swroot}/coptime/settings", \%data) if ( -e "${General::swroot}/coptime/settings" );
}


sub fetch_data()
{
  # collect uptimer data
  $data{'IPCOP_VERSION'} = $General::version;
  $data{'OS_NAME'} = `uname`;
  $data{'OS_VERSION'} = `uname -r`;
  $data{'MACHINE'} = `uname -m`;
}
